indexing
	descripcion: "Describen el TAD que representa los números complejos"
class COMPLEJO
creation
	coordenadas_rectangulares, coordenadas_polares
feature {NONE}
	coordenadas_rectangulares(a,b: DOUBLE) is
	-- permite crear un número complejo cuando se dispone como dato su representación binomial
	-- a es la parte real
	-- b es la parte imaginaria
	do
		real := a
		img  := b
	end

	coordenadas_polares(a,b: DOUBLE) is
	-- permite crear un número complejo cuando se dispone como dato su representación trigonométrica o polar
	-- a es la distancia del origen al punto, 
	-- b es el ángulo de inclinación del punto a partir del eje X.
	do
		real := a * b.cos
		img  := a * b.sin
	end

feature {ANY}

	real: DOUBLE -- parte real, representación binomial
	
	img: DOUBLE -- parte imaginaria, representación binomial

	rho: DOUBLE is -- distancia del origen al punto, representación trigonométrica o polar
	do
		Result := abs
	end
	
	theta: DOUBLE is -- ángulo de inclinación del punto a partir del eje X, representación trigonométrica o polar
	do
		if real/= 0 then
			Result := (img/real).atan
		else 
			Result := 3.1416 / 2
		end
	end

	infix "+" (otro: COMPLEJO): COMPLEJO is
	-- obtiene un nuevo número complejo resultado de sumar Current y otro
	require
		otro_valido: otro /= Void
	do
		create Result.coordenadas_rectangulares(real + otro.real, img + otro.img)
	end

	infix "-" (otro: COMPLEJO): COMPLEJO is
	-- obtiene un nuevo número complejo resultado de restar Current y otro
	require
		otro_valido: otro /= Void
	do
		create Result.coordenadas_rectangulares(real - otro.real, img - otro.img)
	end

	infix "&" (otro: COMPLEJO): COMPLEJO is
	-- obtiene un nuevo número complejo resultado de multiplicar Current y otro, aplica la operación de forma trigonométrica,
	-- mediante la multiplicación de las distancias y la suma de los ángulos.
	require
		otro_valido: otro /= Void
	do
		create Result.coordenadas_polares(rho*otro.rho, theta+otro.theta)
	end

	infix "*" (otro: COMPLEJO): COMPLEJO is
	-- obtiene un nuevo número complejo resultado de multiplicar Current y otro
	require
		otro_valido: otro /= Void
	do
		create Result.coordenadas_rectangulares(real*otro.real - img*otro.img, 
							real*otro.img + img*otro.real)
	end

	infix "/" (otro: COMPLEJO): COMPLEJO is
	-- obtiene un nuevo número complejo resultado de dividir Current y otro
	require
		otro_valido: otro /= Void
		cociente_valido: otro.conjugado.abs /= 0
	do
		create Result.coordenadas_rectangulares(  (real*otro.real + img*otro.img)/otro.abs^2, (-real*otro.img + img*otro.real)/otro.abs^2 )
	end

	conjugado: COMPLEJO is
	-- obtiene un nuevo número complejo que es el conjugado del Current
	do
		create Result.coordenadas_rectangulares(real, -img)
	end

	abs: DOUBLE is
	-- obtiene el módulo o valor absoluto del número complejo
	do
		Result := (real^2 + img^2).sqrt
	end

	to_string: STRING is
	-- obtiene un string que contiene la representación binomial del número complejo
	do
		Result := "( " + real.to_string + " + " + img.to_string + "i )"
	end

	to_string_polares: STRING is
	-- obtiene un string que contiene la representación trigonométrica o polar del número complejo
	do
		Result := rho.to_string + "cis" + theta.to_string
	end

end--class COMPLEJO
