indexing
	descripcion: "Representa un punto en el plano."
	comentarios: "Se tiene su representación en coordenadas cartesianas x,y, y en coordenadas polares rho y theta"
class PUNTO
creation
	crea_polares, crea_cartesianas
feature

   x: DOUBLE -- Parte x de la representación del punto en coordenadas cartesianas

   y: DOUBLE -- Parte y de la representación del punto en coordenadas cartesianas

   rho: DOUBLE is
   -- Distancia del origen, componente rho del punto en coordenadas polares
      do
	 Result := x ^ 2 + y ^ 2
	 Result := Result.sqrt
      end

   theta: DOUBLE is
   -- Angulo respecto del origen de coordenadas, componente theta del punto en coordenadas polares
      do
	 if x = 0 then
	    Result := 3.1416 / 2
	 else
	    Result := (y / x).atan
	 end
      end

   infix "-", distancia(p: PUNTO): DOUBLE is
   -- Se obtiene la distancia del punto a otro punto dado (p)
      do
	 Result := ((x - p.x) ^ 2 + (y - p.y) ^ 2).sqrt
      end

   escala(f: DOUBLE) is
   -- Se transforma el punto mediante un escalado, tomando f como factor.
      do
	 x := x * f
	 y := y * f
      end

   traslada(a, b: DOUBLE) is
   -- Se transforma el punto mediante una traslación, tomando a y b como incrementos en los ejes X e Y
      do
	 x := x + a
	 y := y + b
      end

   rota(p: PUNTO; a: DOUBLE) is
   -- Se transforma el punto mediante una rotación alrededor del punto dado (p) con el ángulo dado (a)
   -- Se apoya en la definición de rota_centro, aplicando un traslado hacia el origen, aplicando la rotación
   -- y posteriormente devolver el traslado realizado.
   local
       cuanto_x, cuanto_y: INTEGER
      do
      	 cuanto_x := p.x
	 cuanto_y := p.y
	 Current.traslada(- cuanto_x, - cuanto_y)
	 Current.rota_centro(a)
	 Current.traslada(cuanto_x, cuanto_y)
      end

   rota_centro(a: DOUBLE) is
   -- Se transforma el punto mediante una rotación alrededor del origen del sistema de coordenadas con el ángulo dado (b)
      do
	 x := x * a.cos + y * a.sin
	 y := x * a.sin + y * a.cos
      end

   to_string: STRING is
   -- Obtiene una representación en cadena de caracteres del punto, teniendo en cuenta sus coordenadas cartesiandas
      do
	 Result := "(" + x.to_string + "," + y.to_string + ")"
      end

   to_string_en_polares: STRING is
   -- Obtiene una representación del punto  en cadena de caracteres, teniendo en cuenta sus coordenadas polares
      do
	 Result := "polares:<" + rho.to_string + "," + theta.to_string + ">"
      end

feature {NONE}
   crea_polares(a,b: DOUBLE) is
   -- Crea un objeto punto a partir de sus datos representando las coordenadas
   -- polares del punto. 
   -- "a" representa la distancia desde el origen al punto
   -- "b" representa el ángulo
   do
   	x := a * b.cos
	y := a * b.sin
   end

   crea_cartesianas(a,b: DOUBLE) is
   -- Crea un objeto punto a partir de sus datos representando las coordenadas
   -- cartesianas del punto.
   -- "a" representa el desplazamiento en el Eje X (horizontal)
   -- "b" representa el desplazamiento en el Eje Y (vertical)
   do
   	x := a
	y := b
   end

end -- class PUNTO
