indexing
	descripcion: "Modela un usuario del sistema."
	datos: "Almacena, el identificador, el nombre, los apellidos y el tipo de usuario."
	casos_especiales: "Cuando el nombre del usuario es una sola palabra o la cadena vacía, indicará tipo de usuario: sistema"
	comentario: "Se corresponde con un fragmento del enunciado del práctica del curso 2005-2006"
	version: "Puede haber muchos caminos para llegar a la solución. Se opta por un camino para ilustrar diferentes funcionalidades de la clase STRING"
class USUARIO
creation
	make_from_string
feature
	nombre: STRING
	apellidos: STRING
	tipo_de_usuario: STRING
	id: STRING

feature {NONE} -- público como método de creación, pero no puede ser usado como un método normal

	make_from_string(origen: STRING) is
	-- Crea un objeto a partir de la información contenida en una línea del fichero /etc/passwd
	-- requiere que el string enviado no contenga cambios de línea
	-- es decir que se corresponda con una línea 
	-- y que contenga tantos campos separados por ':' como una línea del fichero /etc/passwd
	require 
		origen_valido: origen /= Void
		no_cambios_de_linea: origen.occurrences('%N')=0
		suficientes_datos: origen.occurrences(':') = 6
	local
		partes: ARRAY[STRING]
		copia_interna: STRING

	do
		-- Para no modificar la cadena que se ha obtenido como parámetro
		copia_interna := clone(origen)

		-- preparar el string de entrada manejado a través de una copia interna
		prepara_cadena(copia_interna)

		-- Se obtiene un array. En cada casilla del array el contenido separado entre ":" en la cadena original
		partes := copia_interna.split

		-- El id del usuario está en el campo número 1 de la cadena original
		id := clone(partes.item(1))

		-- El nombre completo y el tipo del usuario están en el campo número 5 de la cadena original
		obtener_datos(partes.item(5))

		-- Si nombre de usuario es vacío, se usa el primer campo
		if nombre.is_equal("") then
			nombre := clone(partes.item(1))
		end
	end -- make_from_string

feature {NONE} -- métodos auxiliares

	prepara_cadena(cadena: STRING) is
	-- preparar el string de entrada manejado a través de una copia interna
        -- se hará una partición del mismo descansando en el concepto de separador
        -- para ello los ":" se convertirán en cambios de línea
        -- cuando se encuentren los : seguidos se necesitará introducir un carácter intermedio para detectar el caso
	do
		-- Se cambia la presencia de "::" por ":#"
		from
		until
			not cadena.valid_index(cadena.substring_index("::",1)) -- no se encuentra "::" en la cadena
		loop
			cadena.replace_substring(":#:",cadena.substring_index("::",1),cadena.substring_index("::",1)+1)
		end

		-- convierte ':' en cambios de línea
		cadena.replace_all(':','%N')
		-- Se elimina la presencia de otros separadores que no sean cambios de línea. De esta forma
		-- se obtendrá la partición de la cadena de partida únicamente por los ":" que tenía originalmente
		cadena.replace_all(' ','#')
		cadena.replace_all('	','#')
	end -- prepara_cadena

	obtener_datos(cadena: STRING) is
	-- Obtiene datos de la cadena para dar valor a los atributos nombre, apellidos y tipo_de_usuario
	local
		partes_nombre: ARRAY[STRING]
		palabra: STRING
		i : INTEGER
	do
		nombre := ""
		apellidos := "" 

		-- si nombre de usuario no existe. 
		if cadena.is_equal("#") then
			-- Tipo de usuario especial
			tipo_de_usuario := "sistema"
		else
			-- Se vuelven a introducir espacios en lugar del carácter #
			cadena.replace_all('#',' ')
			-- Se separan los datos contenidos en esta cadena, los únicos separadores son los espacios
			-- El array contiene en cada casilla una palabra
			partes_nombre := cadena.split

			-- usuario de tipo sistema
			if partes_nombre.count = 1 then
				nombre := clone(partes_nombre.item(1))
				tipo_de_usuario := "sistema"
			else
				-- capturar el nombre y el tipo de usuario
				-- La última palabra contiene el nombre y el tipo del usuario separado por '-'
				palabra := partes_nombre.item(partes_nombre.count)

				if not palabra.valid_index(palabra.first_index_of('-')-1) then
					tipo_de_usuario := "sistema"
				else 
					-- Obtiene el nombre del usuario
					nombre := clone(palabra.substring(1,palabra.first_index_of('-')-1))
					palabra.remove_substring(1,palabra.first_index_of('-')-1)

					palabra.remove_all_occurrences('-')

					-- Obtiene el tipo de usuario
					tipo_de_usuario := clone(palabra)

					-- Obtener la cadena que contiene los apellidos. Los apellidos podrían estar formados por varias palabras
					-- Se obtienen concatenando todas las palabras menos la última que contenía el nombre y el tipo de usuario
					-- Crea un string vacío
					create apellidos.make_empty
					from
						i := 1
					until 
						i = partes_nombre.count  -- todas las palabras menos la última
					loop
						apellidos.append(partes_nombre.item(i))  -- añade al final de la cadena
						i := i + 1	
						if i /= partes_nombre.count then
							apellidos.append(" ") -- inserta espacio entre ellas
						end
					end
					--limpia de la cadena apellidos la ocurrencia de comas
					apellidos.remove_all_occurrences(',')
				end
			end
		end 
	end -- obtener_datos
end -- class USUARIO
