indexing
descripcion: "Clase que representa una estructura de almacenamiento tipo tabla"
parametro_k: "Un tipo para las claves para acceder a los elementos en la tabla"
parametro_g: "Un tipo para los elementos que se almacenan en la tabla"
ejemplos: "TABLA[INTEGER,INTEGER], TABLA[CHARACTER, STRING], TABLA[STRING,USUARIO]"
class TABLA[K,G]
creation
	crea_vacia
feature -- Interfaz de Consulta
	claves: ARRAY[K] is
	-- todas las claves que permiten localizar alguna 
	-- entrada en la tabla
	do
		Result := clone(lista_claves)
	end

	valores: ARRAY[G] is
	-- todos los valores almacenados en la tabla, localizables
	-- a través de alguna clave
	do
		Result := clone(lista_valores)
	end

	primera_ocurrencia_de_valor(i: K): G is
	-- obtiene un elemento almacenado accesible a través de la clave
	-- dada por i (el primero que se encuentre) 
	-- Las claves las localiza por valor (busca con has, index_of)
	require
		esta: claves.has(i)
	do
		Result :=  lista_valores.item(lista_claves.index_of(i)) 
	end

	primera_ocurrencia_de_clave(v: G): K is
	-- obtiene la primera clave que se encuentre a partir de la cual
	-- se accede al elemento v
	-- Los valores los localiza por referencia (busca con fast_has, fast_index_of)
	require
		esta: valores.fast_has(v)
	do
		Result := lista_claves.item(lista_valores.fast_index_of(v))
	end

	valores_con_igual_clave(i: K): ARRAY[G] is
	-- obtiene todos los elementos almacenados bajo 
	-- la misma clave (colisiones)
	-- el array estará vacío si no existe la clave en la tabla
	-- Las claves las localiza por valor (compara con equal)
	local
		pos : INTEGER
	do
		create Result.make(1,0)
		from
			pos := lista_claves.lower
		until
			pos > lista_claves.upper
		loop	
			if equal(lista_claves.item(pos),i) then
				Result.add_last(lista_valores.item(pos))
			end
			pos := pos + 1
		end
	end

	claves_con_igual_valor(v: G): ARRAY[K] is
	-- obtiene todas las claves a través de las cuales
	-- se puede acceder al mismo valor (aliases)
	-- el array estará vacío si no existe el elemento almacenado en la tabla
	-- Los valores los localiza por referencia (compara con "=")
	local
		pos : INTEGER
	do
		create Result.make(1,0)
		from
			pos := lista_valores.lower
		until
			pos > lista_valores.upper
		loop	
			if lista_valores.item(pos)=v then
				Result.add_last(lista_claves.item(pos))
			end
			pos := pos + 1
		end
	end

feature -- Interfaz de modificación

	insertar(i: K; v: G) is
	-- almacena el elemento v que podrá ser obtenido a través de la clave i
	do
		lista_claves.add_last(i)
		lista_valores.add_last(v)
	end

	crea_vacia is
	-- obtiene una tabla vacía
	do
		create lista_claves.make(1,0)
		create lista_valores.make(1,0)
	end

feature {NONE} -- Implementación

	lista_claves: ARRAY[K]
	lista_valores: ARRAY[G]

end--class TABLA[K,G]
